@@ -3,7 +3,7 @@
 """
 Utility to help downloading spice kernels.
 
-Will download local copy of kernels with same directory structure as on 
+Will download local copy of kernels with same directory structure as on
 https://naif.jpl.nasa.gov/. Use :func:`planetmapper.set_kernel_path` to choose the
 location that the kernels are downloaded to.
 
@@ -24,6 +24,7 @@ These functions can be used to download a set of URLS. For example: ::
     )
 
 """
+
 import os
 import urllib.parse
 import urllib.request
@@ -36,7 +37,7 @@ from .base import get_kernel_path
 URL_ROOT = 'https://naif.jpl.nasa.gov/pub/'
 
 
-def download_urls(*urls: str) -> None:
+def download_urls(*urls: str, **kwargs) -> None:
     """
     Download data from naif.jpl.nasa.gov and save locally.
 
@@ -50,17 +51,19 @@ def download_urls(*urls: str) -> None:
 
     Args:
         urls: kernel URL on naif.jpl.nasa.gov.
+        **kwargs: passed to :func:`download_kernel` and
+            :func:`download_kernels_from_webpage`.
     """
     for url in urls:
         # look for '.' in filename part of url to identify if a file/directory
         path = urllib.parse.urlsplit(url).path
         if '.' in os.path.split(path)[1]:
-            download_kernel(url)
+            download_kernel(url, **kwargs)
         else:
-            download_kernels_from_webpage(url)
+            download_kernels_from_webpage(url, **kwargs)
 
 
-def download_kernels_from_webpage(index_url: str) -> None:
+def download_kernels_from_webpage(index_url: str, **kwargs) -> None:
     """
     Download all first-level kernels listed in the page given by index_url.
 
@@ -74,11 +77,12 @@ def download_kernels_from_webpage(index_url: str) -> None:
 
     Args:
         index_url: URL of index page on naif.jpl.nasa.gov.
+        **kwargs: passed to :func:`download_kernel`.
     """
     urls = get_kernel_paths_from_webpage(index_url)
     print(f'{len(urls)} to download from {index_url}')
     for idx, url in enumerate(urls):
-        download_kernel(url, note=f'[{idx+1}/{len(urls)}] ')
+        download_kernel(url, note=f'[{idx + 1}/{len(urls)}] ', **kwargs)
     print(f'All kernels downloaded from {index_url}')
     print()
 
@@ -102,7 +106,7 @@ def download_kernel(url: str, force_download: bool = False, note: str = '') -> N
         if force_download:
             print('  Kernel already exists, downloading anyway')
         else:
-            print('  ✅ Kernel already exists locally')
+            print('  OK - Kernel already exists locally')
             return
     local_path = _convert_url_to_local_path(url)
     print(f'  Downloading to {local_path}')
@@ -122,6 +126,7 @@ def get_kernel_paths_from_webpage(index_url: str) -> list[str]:
     Returns:
         List of URL strings corresponding to kernels on the webpage.
     """
+    # pylint: disable=consider-using-with
     assert index_url.startswith(URL_ROOT), f'URL must begin with {URL_ROOT}'
     webpage = urllib.request.urlopen(index_url).read().decode()
     data = webpage.split('<!--start data_content-->')[1].split('</table>')[0]
@@ -174,7 +179,7 @@ def _get_kernel_path(p: str) -> str:
 
 def _kernel_path_to_url(kp: str) -> str:
     """Create URL from a kernel path"""
-    return URL_ROOT + os.path.sep + kp
+    return URL_ROOT + kp
 
 
 def _kernel_path_to_local_path(kp: str) -> str:
