@@ -1,29 +1,33 @@
-from functools import wraps
-from typing import TypeVar, ParamSpec, Concatenate, Callable, TYPE_CHECKING
-import tqdm
 import time
-import numpy as np
-import warnings
+from abc import ABC, abstractmethod
+from functools import wraps
+from typing import TYPE_CHECKING, Callable, Concatenate, ParamSpec, TypeVar
+
+import tqdm
+
 if TYPE_CHECKING:
     from .base import SpiceBase
 
 T = TypeVar('T')
-S = TypeVar('S')
+S = TypeVar('S', bound='SpiceBase')
 P = ParamSpec('P')
 
 
 def progress_decorator(
-    fn: Callable[Concatenate[S, P], T]
+    fn: Callable[Concatenate[S, P], T],
 ) -> Callable[Concatenate[S, P], T]:
+    """Decorator for functions which track progress using progress hooks."""
+
+    # pylint: disable=protected-access
     @wraps(fn)
-    def decorated(self: 'SpiceBase', *args: P.args, **kwargs: P.kwargs):
+    def decorated(self: S, *args: P.args, **kwargs: P.kwargs) -> T:
         if self._progress_hook is None:
-            return fn(self, *args, **kwargs)  # type: ignore
+            return fn(self, *args, **kwargs)
         else:
             self._progress_call_stack.append(fn.__qualname__)
             self._update_progress_hook(0)
             try:
-                out = fn(self, *args, **kwargs)  # type: ignore
+                out = fn(self, *args, **kwargs)
             except:
                 self._progress_call_stack.pop()
                 raise
@@ -34,12 +38,14 @@ def progress_decorator(
                 pass
             return out
 
-    return decorated  # type: ignore
+    return decorated
 
 
-class ProgressHook:
-    def __call__(self, progress: float, stack: list[str]) -> None:
-        raise NotImplementedError
+class ProgressHook(ABC):
+    """Base class for progress hooks."""
+
+    @abstractmethod
+    def __call__(self, progress: float, stack: list[str]) -> None: ...
 
 
 class CLIProgressHook(ProgressHook):
@@ -47,8 +53,8 @@ class CLIProgressHook(ProgressHook):
     General progress hook to display progress of each decorated function individually.
     """
 
-    def __init__(self, leave: bool | None = None, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+    def __init__(self, leave: bool | None = None, **kwargs) -> None:
+        super().__init__(**kwargs)
         self.leave = leave
         self.bars: dict[tuple[str, ...], tqdm.tqdm] = {}
 
@@ -90,14 +96,14 @@ class TotalTimingProgressHook(ProgressHook):
             self_duration = full_diration
             for k, v in self.self_durations.items():
                 if len(k) > len(key) and k[: len(key)] == key:
-                    self_duration -= self.self_durations[k]
+                    self_duration -= v
             self.self_durations[key] = self_duration
             desc = '> ' * (len(key) - 1) + key[-1]
             print(f'{full_diration:5.2f} {self_duration:5.2f}  {desc}')
 
 
 # Specific progress hooks for use in saving
-class SaveProgressHook(ProgressHook):
+class _SaveProgressHook(ProgressHook):
     """
     Base class for progress hook to use when saving data.
 
@@ -139,14 +145,14 @@ class SaveProgressHook(ProgressHook):
         self.progress_parts[key] = progress
         self.update_bar(progress_change)
 
-    def update_bar(self, progress_change: float) -> None:
-        raise NotImplementedError
+    @abstractmethod
+    def update_bar(self, progress_change: float) -> None: ...
 
-    def get_description(self) -> str:
-        return ''
+    @abstractmethod
+    def get_description(self) -> str: ...
 
 
-class SaveNavProgressHook(SaveProgressHook):
+class _SaveNavProgressHook(_SaveProgressHook):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self.weights: dict[str, float] = {
@@ -155,8 +161,10 @@ class SaveNavProgressHook(SaveProgressHook):
             'BodyXY._get_radec_img': 5,
             'BodyXY._get_lonlat_centric_img': 5,
             'BodyXY._get_illumination_gie_img': 50,
+            'BodyXY.get_local_solar_time_img': 5,
             'BodyXY._get_state_imgs': 30,
             'BodyXY.get_radial_velocity_img': 5,
+            'BodyXY._get_limb_coordinate_imgs': 20,
             'BodyXY._get_ring_coordinate_imgs': 50,
             'Observation.save_observation': 100,
         }
@@ -166,7 +174,7 @@ class SaveNavProgressHook(SaveProgressHook):
         return 'Saving observation'
 
 
-class SaveMapProgressHook(SaveProgressHook):
+class _SaveMapProgressHook(_SaveProgressHook):
     def __init__(self, n_wavelengths: int, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self.n_wavelengths = n_wavelengths
@@ -177,8 +185,10 @@ class SaveMapProgressHook(SaveProgressHook):
             'BodyXY._get_radec_map': 10,
             'BodyXY._get_xy_map': 10,
             # 'BodyXY._get_lonlat_map': 10,
+            'BodyXY.get_local_solar_time_map': 5,
             'BodyXY._get_lonlat_centric_map': 5,
             'BodyXY._get_state_maps': 5,
+            'BodyXY._get_limb_coordinate_maps': 10,
             'BodyXY._get_ring_coordinate_maps': 10,
             'Observation.save_mapped_observation': 20,
         }
@@ -188,7 +198,7 @@ class SaveMapProgressHook(SaveProgressHook):
         return 'Saving map'
 
 
-class SaveProgressHookCLI(SaveProgressHook):
+class _SaveProgressHookCLI(_SaveProgressHook):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self.bar = tqdm.tqdm(
@@ -200,18 +210,16 @@ class SaveProgressHookCLI(SaveProgressHook):
         )
 
     def update_bar(self, progress_change: float) -> None:
-        # * 0.99999 to ensure floating point errros don't accumulate to >100%
-        self.bar.update(progress_change * 100 *0.99999)
+        # * 0.99999 to ensure floating point errros don't accumulate to >100%
+        self.bar.update(progress_change * 100 * 0.99999)
         if self.progress_parts.get(self.default_key, 0) >= 1:
             self.bar.update(100 - self.overall_progress * 100)
             self.bar.close()
 
 
-class SaveNavProgressHookCLI(SaveNavProgressHook, SaveProgressHookCLI):
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+class SaveNavProgressHookCLI(_SaveNavProgressHook, _SaveProgressHookCLI):
+    pass
 
 
-class SaveMapProgressHookCLI(SaveMapProgressHook, SaveProgressHookCLI):
-    def __init__(self, n_wavelengths: int, *args, **kwargs) -> None:
-        super().__init__(n_wavelengths, *args, **kwargs)
+class SaveMapProgressHookCLI(_SaveMapProgressHook, _SaveProgressHookCLI):
+    pass
