@@ -1,15 +1,15 @@
 """
 Various general helpful utilities.
 """
+
 import os
 import pathlib
-import traceback
 import warnings
-from datetime import datetime
-from typing import Literal
+from typing import Literal, Sequence
 
 import matplotlib.ticker
 import numpy as np
+from astropy.io import fits
 from matplotlib.axes import Axes
 
 
@@ -18,7 +18,7 @@ def format_radec_axes(
     dec: float,
     dms_ticks: bool = True,
     add_axis_labels: bool = True,
-    aspect_adjustable: Literal['box', 'datalim'] = 'datalim',
+    aspect_adjustable: Literal['box', 'datalim'] | None = 'datalim',
 ) -> None:
     """
     Format an axis to display RA/Dec coordinates nicely.
@@ -30,12 +30,15 @@ def format_radec_axes(
             (e.g. 12°34′56″) or decimal degrees (e.g. 12.582).
         add_axis_labels: Add axis labels.
         aspect_adjustable: Set `adjustable` parameter when setting the aspect ratio.
-            Passed to :func:`matplotlib.axes.Axes.set_aspect`.
+            Passed to :func:`matplotlib.axes.Axes.set_aspect`. Set to None to skip
+            setting the aspect ratio (generally this is only recommended if you're
+            setting the aspect ratio yourself).
     """
     if add_axis_labels:
         ax.set_xlabel('Right Ascension')
         ax.set_ylabel('Declination')
-    ax.set_aspect(1 / np.cos(np.deg2rad(dec)), adjustable=aspect_adjustable)
+    if aspect_adjustable is not None:
+        ax.set_aspect(1 / np.cos(np.deg2rad(dec)), adjustable=aspect_adjustable)
     if not ax.xaxis_inverted():
         ax.invert_xaxis()
     if dms_ticks:
@@ -64,12 +67,13 @@ class DMSFormatter(matplotlib.ticker.FuncFormatter):
         self.skip_parts = set()
         self.fmt_s = '02.0f'
 
+    # pylint: disable-next=unused-argument
     def _format(self, dd, pos):
         d, m, s = decimal_degrees_to_dms(dd)
         out = []
-        if 'd' not in self.skip_parts or m == 0 and s == 0:
+        if 'd' not in self.skip_parts or (m == 0 and s == 0):
             out.append(f'{d}°')
-        if 'm' not in self.skip_parts or s == 0:
+        if 'm' not in self.skip_parts or ('d' in self.skip_parts and s == 0):
             out.append(f'{m:02.0f}′')
         if 's' not in self.skip_parts:
             out.append(f'{s:{self.fmt_s}}″')
@@ -118,7 +122,7 @@ class DMSFormatter(matplotlib.ticker.FuncFormatter):
             self.fmt_s = '02.0f'
 
         if self.skip_parts == {'d', 'm', 's'}:
-            self.skip_parts = {'d', 'm', 's'}
+            self.skip_parts = set()
         self.set_offset_string(ofs)
         return super().set_locs(locs)
 
@@ -184,9 +188,9 @@ def decimal_degrees_to_dms(decimal_degrees: float) -> tuple[int, int, float]:
     return int(degrees), int(minutes), seconds
 
 
-def decimal_degrees_to_dms_str(decimal_degrees: float, seconds_fmt: str = '') -> str:
+def decimal_degrees_to_dms_str(decimal_degrees: float, seconds_fmt: str = 'g') -> str:
     """
-    Create nicely formated DMS string from decimal degrees value (e.g. `'12°34′56″'`).
+    Create nicely formatted DMS string from decimal degrees value (e.g. `'12°34′56″'`).
 
     Uses :func:`decimal_degrees_to_dms` to perform the conversion.
 
@@ -194,13 +198,34 @@ def decimal_degrees_to_dms_str(decimal_degrees: float, seconds_fmt: str = '') ->
         decimal_degrees: Decimal degrees.
         seconds_fmt: Optionally specify a format string for the seconds part of the
             returned value. For example, `seconds_fmt='.3f'` will fix three decimal
-            places for the fractional part of the seconds value.
+            places for the fractional part of the seconds value. Note that the integral
+            part of the seconds value will always be zero-padded to two digits, so
+            `seconds_fmt='.3f'` will return seconds as e.g. `01.234`.
 
     Returns:
-        String representting the degress, minutes, seconds of the angle.
+        String representing the degrees, minutes, seconds of the angle.
     """
     d, m, s = decimal_degrees_to_dms(decimal_degrees)
-    return f'{d}°{m}′{s:{seconds_fmt}}″'
+    s_str = f'{s:{seconds_fmt}}'
+    if len(s_str.split('.')[0]) < 2:
+        s_str = '0' + s_str  # Zero pad integer part of seconds to e.g. 01.234
+    return f'{d}°{m:02d}′{s_str}″'
+
+
+class ignore_warnings(warnings.catch_warnings):
+    """
+    Context manager to ignore general warnings using warnings.filterwarnings.
+    """
+
+    def __init__(self, *warning_strings: str, **kwargs):
+        super().__init__(**kwargs)
+        self.warning_strings = warning_strings
+
+    def __enter__(self):
+        out = super().__enter__()
+        for ws in self.warning_strings:
+            warnings.filterwarnings('ignore', ws)
+        return out
 
 
 class filter_fits_comment_warning(warnings.catch_warnings):
@@ -218,159 +243,23 @@ class filter_fits_comment_warning(warnings.catch_warnings):
         return out
 
 
-def cprint(*msg, fg=None, bg=None, style=None, skip_print=False, sep=' ', **kwargs):
-    """
-    Prints coloured and formatted text.
-
-    Parameters
-    ----------
-    msg
-        Message to print.
-
-    fg, bg : {'k', 'r', 'g', 'b', 'y', 'm', 'c', 'w'}
-        Foreground and background colours (see code for options).
-
-    style : {'b', 'f', 'i', 'u', 'x', 'y', 'r', 'h', 's'}
-        Formatting style to apply to text. Can be multiple values, e.g. 'bi'
-        for bold and italic style.
-
-    """
-    colcode = {
-        'k': 0,  # black
-        'r': 1,  # red
-        'g': 2,  # green
-        'y': 3,  # yellow
-        'b': 4,  # blue
-        'm': 5,  # magenta
-        'c': 6,  # cyan
-        'w': 7,  # white
-    }
-
-    fmtcode = {
-        'b': 1,  # bold
-        'f': 2,  # faint
-        'i': 3,  # italic
-        'u': 4,  # underline
-        'x': 5,  # blinking
-        'y': 6,  # fast blinking
-        'r': 7,  # reverse
-        'h': 8,  # hide
-        's': 9,  # strikethrough
-    }
-
-    # Properties
-    props = []
-    if isinstance(style, str):
-        props = [fmtcode[s] for s in style]
-    if isinstance(fg, str):
-        props.append(30 + colcode[fg])
-    if isinstance(bg, str):
-        props.append(40 + colcode[bg])
-
-    # Display
-    msg = sep.join(
-        str(x) for x in msg
-    )  # Reproduce print() behaviour for easy translation
-    props = ';'.join(str(x) for x in props)
-
-    if props:
-        msg = '\x1b[%sm%s\x1b[0m' % (props, msg)
-
-    if not skip_print:
-        print(msg, **kwargs)
-
-    return msg
-
-
-def print_progress(annotation=None, c1='g', c2='k', style='b'):
-    """
-    Print progress summary of current code.
-
-    Prints summary of code location and execution time for use in optimising and monitoring code.
-    Uses traceback to identify the call stack and prints tree-like diagram of stacks where this
-    function was called. The call stack is relative to the first time this function is called as it
-    uses properties of print_progress to communicate between calls.
-
-    Printed output contains:
-        - Seconds elapsed since last call of print_progress.
-        - Current time.
-        - Traceback for location print_progress is called from, relative to first location
-          print_progress was called. This includes file names, function names and line numbers.
-        - Optional annotation provided to explain what current line is.
-
-    Arguments
-    ---------
-    annotation : str
-        Optionally provide annotation about current line of code.
-
-    c1, c2, style : str
-        Formatting options to pass to cprint()
-    """
-    now = datetime.now()  # Ignore duration of current code
-
-    # Get timings
-    if print_progress.last_dtm is None:
-        title = ' seconds'
-    else:
-        td = now - print_progress.last_dtm
-        title = f'{td.total_seconds():8.2f}'
-    title += ' @ '
-    title += now.strftime('%H:%M:%S')
-    title += ' '
-
-    # Get stack
-    stack = traceback.extract_stack()[:-1]
-    if print_progress.first_stack is None:
-        print_progress.first_stack = stack
-    first_stack = print_progress.first_stack
-    split_idx = len([None for a, b in zip(stack[:-1], first_stack) if a == b])
-    stack = stack[split_idx:]
-    stack_parts = [(s[2], s[1], os.path.split(s[0])[1]) for s in stack]
-    stack_text = [
-        f'{a} (line {b} in <ipython>)'
-        if c.startswith('<ipython-input-')
-        else f'{a} (line {b} in {c})'
-        for a, b, c in stack_parts
-    ]
-    last_stack = print_progress.last_stack
-    if last_stack is not None:
-        last_idx = len([None for a, b in zip(stack[:-1], last_stack) if a == b])
-        stack_text = stack_text[last_idx:]
-    else:
-        last_idx = 0
-    print_progress.last_stack = stack
-
-    # Do actual printing
-    for st in stack_text:
-        msg = cprint(
-            ' ' + '│  ' * last_idx + st + ' ',
-            fg=c1,
-            bg=c2,
-            style=style,
-            skip_print=True,
-        )
-        if st == stack_text[-1]:
-            tt = title
-            if annotation:
-                msg += cprint(
-                    f'{annotation} ', fg=c1, bg=c2, style=style + 'i', skip_print=True
-                )
-        else:
-            tt = ' ' * len(title)
-        tt = cprint(tt, fg=c2, bg=c1, style=style, skip_print=True)
-        print(tt + msg, flush=True)
-        last_idx += 1
-    print_progress.last_dtm = datetime.now()  # Ignore duration of this code
-
-
-print_progress.last_dtm = None
-print_progress.last_stack = None
-print_progress.first_stack = None
-
-
-def normalise(values, top=1, bottom=0, single_value=None):
+def normalise(
+    values: np.ndarray | Sequence[float],
+    top: float = 1.0,
+    bottom: float = 0.0,
+    single_value: float | None = None,
+) -> np.ndarray:
     """
     Normalise iterable.
+
+    Args:
+        values: Iterable of values to normalise.
+        top: Top of normalised range.
+        bottom: Bottom of normalised range.
+        single_value: If all values are the same, return this value.
+
+    Returns:
+        Normalised values.
     """
     assert top > bottom
     values = np.array(values)
@@ -381,13 +270,13 @@ def normalise(values, top=1, bottom=0, single_value=None):
 
     # Put into 0 to 1 range
     if vmax != vmin:
-        values = (values - vmin) / (vmax - vmin)
+        values = (values - vmin) / (vmax - vmin)  #  type: ignore
     else:
         values = values - vmin
-    return values * (top - bottom) + bottom
+    return values * (top - bottom) + bottom  #  type: ignore
 
 
-def check_path(path: str):
+def check_path(path: str) -> None:
     """
     Checks if file path's directory tree exists, and creates it if necessary.
 
@@ -405,3 +294,68 @@ def check_path(path: str):
         return
     print('Creating directory path "{}"'.format(path))
     pathlib.Path(path).mkdir(parents=True, exist_ok=True)
+
+
+# Wavelengths
+
+
+class GetWavelengthsError(ValueError):
+    """
+    Error raised when wavelength information cannot be extracted from a FITS header.
+    """
+
+
+def generate_wavelengths_from_header(
+    header: fits.Header | dict,
+    *,
+    check_ctype: bool = True,
+    axis: int = 3,
+) -> np.ndarray:
+    """
+    Generate wavelength array from keyword values in a FITS Header.
+
+    This uses the NAXIS3, CRVAL3, CDELT3 (or CD3_3) and CRPIX3 keywords to generate the
+    wavelength array described by the Header. The axis to generate wavelengths for can
+    be customised using the `axis` parameter.
+
+    By default, this function will raise an exception if the CTYPE of the axis is not
+    'WAVE'. This can be disabled by setting `check_ctype` to False.
+
+    See the
+    `JWST documentation <https://jwst-docs.stsci.edu/jwst-calibration-status/miri-calibration-status/miri-mrs-calibration-status>`_
+    for an an example of how the wavelength array can be generated from the FITS Header.
+
+    Args:
+        header: FITS Header object (or dictionary).
+        check_ctype: Check that the CTYPE of the axis is 'WAVE'.
+        axis: Axis to generate wavelengths for, using FITS (1-based) counting. This
+            defaults to 3.
+
+    Returns:
+        Wavelength array.
+
+    Raises:
+        GetWavelengthsError: If the wavelength array cannot be generated from the
+            FITS Header.
+    """
+    try:
+        if check_ctype and header[f'CTYPE{axis}'] != 'WAVE':
+            raise GetWavelengthsError(
+                f'Header item CTYPE{axis} = {header[f"CTYPE{axis}"]!r} (not \'WAVE\')'
+            )
+
+        naxis3 = int(header[f'NAXIS{axis}'])  # type: ignore
+        crval3 = float(header[f'CRVAL{axis}'])  # type: ignore
+        try:
+            cdelt3 = float(header[f'CDELT{axis}'])  #  type: ignore
+        except KeyError:
+            cdelt3 = float(header[f'CD{axis}_{axis}'])  # type: ignore
+        crpix3 = float(header.get(f'CRPIX{axis}', 1))  #  type: ignore
+    except (KeyError, ValueError, TypeError) as e:
+        raise GetWavelengthsError(
+            'Could not generate wavelength array from FITS Header'
+        ) from e
+
+    # https://jwst-docs.stsci.edu/jwst-calibration-status/miri-calibration-status/miri-mrs-calibration-status
+    wavl = (np.arange(naxis3) + crpix3 - 1) * cdelt3 + crval3
+    return wavl
